/*
 * Copyright (C) 2011 Apple Inc. All Rights Reserved.
 * Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies)
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WebInspectorServer_h
#define WebInspectorServer_h

#if ENABLE(INSPECTOR_SERVER)

#include "WebSocketServer.h"
#include "WebSocketServerClient.h"
#include <wtf/HashMap.h>
#include <wtf/text/WTFString.h>

#if !PLATFORM(WIIU)
#include <string>
#endif

namespace WebCore {

class InspectorClient;

class WebInspectorServer : public WebSocketServer, public WebSocketServerClient {
public:
    typedef HashMap<unsigned, WebCore::InspectorClient*> ClientMap;

    static WebInspectorServer& shared();

	inline static bool HasInstance() {return (s_pInstance != 0);}
    static void create();
    static void destroy(const bool vForceDropSingletonRef = false);

	static void ResetGlobals();

    #if !PLATFORM(WIIU)
    // Set the base path for loading inspector assets
    static void setAssetsPath(const char *set);
    static std::string prependAssetsPath(const char *path);
    #endif

    // Page registry to manage known pages.
    int registerPage(WebCore::InspectorClient* client);
    void unregisterPage(int pageId);
    void sendMessageOverConnection(unsigned pageIdForConnection, const String& message);
    //CRANK-ME WiiU Specific funtion
    void setFrontendUrl(String & frontendUrl, String & hostUrl);

    // Added for RestartWebViews -LH-NSA 11/9/2012
    String getFrontendUrl();

	static void addReference(){s_refCnt++;}
	static void removeReference(){destroy();}

private:
    WebInspectorServer();
    ~WebInspectorServer();

    // WebSocketServerClient implementation. Events coming from remote connections.
    virtual void didReceiveUnrecognizedHTTPRequest(WebSocketServerConnection*, PassRefPtr<WebCore::HTTPRequest>);
    virtual bool didReceiveWebSocketUpgradeHTTPRequest(WebSocketServerConnection*, PassRefPtr<WebCore::HTTPRequest>);
    virtual void didEstablishWebSocketConnection(WebSocketServerConnection*, PassRefPtr<WebCore::HTTPRequest>);
    virtual void didReceiveWebSocketMessage(WebSocketServerConnection*, const String& message);
    virtual void didCloseWebSocketConnection(WebSocketServerConnection*);

    bool platformResourceForPath(const String& path, Vector<char>& data, String& contentType);
//#if PLATFORM(QT)
    void buildPageList(Vector<char>& data, String& contentType);
//#endif

    void closeConnection(InspectorClient*, WebSocketServerConnection*);

    unsigned m_nextAvailablePageId;
    ClientMap m_clientMap;
    HashMap<unsigned, WebSocketServerConnection*> m_connectionMap;

    //WiiU Specific
    String m_frontendUrl;
    String m_hostUrl;

	static WebInspectorServer *	s_pInstance;
	static int					s_refCnt;
};

}

#endif // ENABLE(INSPECTOR_SERVER)

#endif // WebInspectorServer_h
